﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Drawing2D;
using System.Drawing.Text;

namespace howto_navigate_tree
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // The root node.
        private TreeNode<CircleNode> Node0 =
            new TreeNode<CircleNode>(new CircleNode("0"));

        // Make a tree.
        private void Form1_Load(object sender, EventArgs e)
        {
            TreeNode<CircleNode> node1 =
                new TreeNode<CircleNode>(new CircleNode("1"));
            TreeNode<CircleNode> node2 =
                new TreeNode<CircleNode>(new CircleNode("2"));
            TreeNode<CircleNode> node3 =
                new TreeNode<CircleNode>(new CircleNode("3"));
            TreeNode<CircleNode> node4 =
                new TreeNode<CircleNode>(new CircleNode("4"));
            TreeNode<CircleNode> node5 =
                new TreeNode<CircleNode>(new CircleNode("5"));
            TreeNode<CircleNode> node6 =
                new TreeNode<CircleNode>(new CircleNode("6"));
            TreeNode<CircleNode> node10 =
                new TreeNode<CircleNode>(new CircleNode("10"));
            TreeNode<CircleNode> node13 =
                new TreeNode<CircleNode>(new CircleNode("13"));

            Node0.LeftChild = node1;
            Node0.RightChild = node2;
            node1.LeftChild = node3;
            node1.RightChild = node4;
            node2.LeftChild = node5;
            node2.RightChild = node6;
            node4.RightChild = node10;
            node6.LeftChild = node13;

            // Arrange the tree.
            ArrangeTree();
        }

        // Draw the tree.
        private void picTree_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;
            e.Graphics.TextRenderingHint = TextRenderingHint.AntiAliasGridFit;
            Node0.DrawTree(e.Graphics);
        }

        // Center the tree on the form.
        private void picTree_Resize(object sender, EventArgs e)
        {
            ArrangeTree();
        }
        private void ArrangeTree()
        {
            using (Graphics gr = picTree.CreateGraphics())
            {
                // Arrange the tree once to see how big it is.
                float xmin = 0, ymin = 0;
                Node0.Arrange(gr, ref xmin, ref ymin);

                // Arrange the tree again to center it horizontally.
                xmin = (this.ClientSize.Width - xmin) / 2;
                ymin = 10;
                Node0.Arrange(gr, ref xmin, ref ymin);
            }

            picTree.Refresh();
        }

        // The currently selected node.
        private TreeNode<CircleNode> SelectedNode;

        // Display the text of the node under the mouse.
        private void picTree_MouseMove(object sender, MouseEventArgs e)
        {
            // Find the node under the mouse.
            FindNodeUnderMouse(e.Location);

            // If there is a node under the mouse,
            // display the node's text.
            if (SelectedNode == null)
            {
                lblNodeText.Text = "";
            }
            else
            {
                lblNodeText.Text = "Node " +
                    SelectedNode.Data.Text + ": " +
                    SelectedNode.NodeNumber();
            }
        }

        // If this is a right button down and the
        // mouse is over a node, display a context menu.
        private void picTree_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button != MouseButtons.Right) return;

            // Find the node under the mouse.
            FindNodeUnderMouse(e.Location);

            // If there is a node under the mouse,
            // display the context menu.
            if (SelectedNode != null)
            {
                // Don't let the user add a child if the
                // selected node already has two children.
                ctxNodeAddLeftChild.Enabled = (SelectedNode.LeftChild == null);
                ctxNodeAddRightChild.Enabled = (SelectedNode.RightChild == null);

                // Don't let the user delete the root node.
                // (The TreeNode class can't do that.)
                ctxNodeDelete.Enabled = (SelectedNode != Node0);

                // Display the context menu.
                ctxNode.Show(this, e.Location);
            }
        }

        // Set SelectedNode to the node under the mouse.
        private void FindNodeUnderMouse(PointF pt)
        {
            using (Graphics gr = picTree.CreateGraphics())
            {
                SelectedNode = Node0.NodeAtPoint(gr, pt);
            }
        }

        // Add a left child to the selected node.
        private void ctxNodeAddLeftChild_Click(object sender, EventArgs e)
        {
            NodeTextDialog dlg = new NodeTextDialog();
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                TreeNode<CircleNode> child =
                    new TreeNode<CircleNode>(new CircleNode(dlg.txtNodeText.Text));
                SelectedNode.LeftChild = child;

                // Rearrange the tree to show the new node.
                ArrangeTree();
            }
        }

        // Add a right child to the selected node.
        private void ctxNodeAddRightChild_Click(object sender, EventArgs e)
        {
            NodeTextDialog dlg = new NodeTextDialog();
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                TreeNode<CircleNode> child =
                    new TreeNode<CircleNode>(new CircleNode(dlg.txtNodeText.Text));
                SelectedNode.RightChild = child;

                // Rearrange the tree to show the new node.
                ArrangeTree();
            }
        }

        // Delete this node from the tree.
        private void ctxNodeDelete_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to delete this node?",
                "Delete Node?", MessageBoxButtons.YesNo,
                MessageBoxIcon.Question) == DialogResult.Yes)
            {
                // Delete the node and its subtree.
                Node0.DeleteNode(SelectedNode);

                // Rearrange the tree to show the new structure.
                ArrangeTree();
            }
        }

        // Find a node by number.
        private void mnuDataFindNode_Click(object sender, EventArgs e)
        {
            NodeTextDialog dlg = new NodeTextDialog();
            dlg.Text = "Enter Node Number";
            dlg.lblPrompt.Text = "Node #:";
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                // Clear any previously selected nodes.
                Node0.SetSubtreeBg(Brushes.White);

                // Find the indicated node.
                int number = int.Parse(dlg.txtNodeText.Text);
                TreeNode<CircleNode> selected_node =
                    Node0.FindNodeByNumber(number);

                // Mark the node.
                if (selected_node != null)
                    selected_node.BgBrush = Brushes.Pink;

                // Rearrange the tree to show the new node.
                ArrangeTree();
            }
        }
    }
}
